import os
import pandas as pd
import json
from .database.mycosmosdb import callConnect
from .endpoints.utils import read_item
from .endpoints.measurements import post_custom_measure
from .endpoints.measurements import PostData

# this script will clear the response from the Ownership/Transaction Details
# this was done because of a bug where the default was "false"
# so it'll only clear if the response is an exact match for the string "false"
# it'll use the same table from EIP Historical Impact Data.xlsx to fetch the accounts

# to use it: python -m app.clearOwnershipResponse

# Set up the path for the Excel file
current_directory = os.path.dirname(os.path.realpath(__file__))
# excel_file_path = os.path.join(current_directory, 'database', 'testData.xlsx')
excel_file_path = os.path.join(current_directory, 'database', 'jurisdictionFix.xlsx')

# Read the Excel file into a pandas DataFrame
df = pd.read_excel(excel_file_path)
df = df.fillna("")
df = df.astype(str)

def find_or_create_section(data, label, structure):
    """
    Recursively finds or creates a section with the given label in the data structure.
    """
    for item in data:
        if item.get('label') == label:
            return item

    # If not found, create a new section with the specified structure
    new_section = {'label': label, **structure}
    data.append(new_section)
    return new_section

def update_json_value(json_data, df_field, replaceWith):
    # Convert the DataFrame value to a string
    value = replaceWith

    # Ensure the 'custommeasure' section exists
    custom_measure = json_data.setdefault('custommeasure', [])

    # Find or create the 'ImpactOverview' section
    impact_overview = find_or_create_section(custom_measure, 'GovernanceImpact', {'metrics': []})

    # Find or create the '2022' metric
    metric_2022 = find_or_create_section(impact_overview['metrics'], '2023', {'value': []})

    # Find or create the 'Q0' value data
    value_data_q0 = find_or_create_section(metric_2022['value'], 'Q0', {'metrics': []})

    # Find or create the '1' q0 metric
    q0_metric_1 = find_or_create_section(value_data_q0['metrics'], '1', {'value': [{}]})

    # Update the value in the JSON data
    if (q0_metric_1['value'][0][df_field] == "false"):
        q0_metric_1['value'][0][df_field] = value

    # return all the responses
    return q0_metric_1['value'][0]


# Function to update JSON data based on DataFrame row
async def update_json_from_df(data, df_row):
    # Mapping of DataFrame columns to JSON fields
    replaceFieldWith = {
        'nonCoopJursidiction': ''
    }

    d = {}
    # Update each field in the JSON data based on the mapping
    for df_field, replaceWith in replaceFieldWith.items():
        d = update_json_value(data, df_field, replaceWith)
    
    print(f"data: {d}")
    await post_custom_measure(PostData(
        oid= data['id'],
        email= data['email'],
        year= "2023",
        quarter= "Q0",
        metric= 'GovernanceImpact',
        data= d
    )) 

# Main code
async def main():
    for index, row in df.iterrows():

        # Retrieve the JSON data from the database for each email
        oid = row['OID']
        # conditional for testing with EIP Test account
        # remove this conditional for actual implementation
        # note that read_item won't work because the third param for EIP test account 
        # should be "info301@metric-esg.com"
        # if oid != '6d5b7e60-43c1-492d-ad9a-f04774e552b7':
        #     print("Skipping iteration due to empty 'oid'")
        #     continue  # Skip this iteration and move to the next one
        print(f"Processing email: {oid}")
        data = read_item(callConnect.container15, oid, oid)

        # Check if the data fetched is a string and convert it to a dictionary
        if isinstance(data, str):
            data = json.loads(data)

        # Update the JSON data with values from the DataFrame row
        await update_json_from_df(data, row)


if __name__ == "__main__":
    import asyncio

    # Run the asynchronous code
    asyncio.run(main())
    
